#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "/usr/src/linux/include/scsi/scsiinfo.h"

/* This is a simple program to test the ioctls of the scsiinfo driver.

*  Copyright (C) 2001 D. Gilbert
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2, or (at your option)
*  any later version.

   Invocation: si_test

   Version 0.12 (20010226)

*/

int main(int argc, char * argv[])
{
    int si_fd, k, res;
    struct scsiinfo_state ss;
    struct scsiinfo_att_list * alp;
    struct scsiinfo_att_dev * adp;
    struct scsiinfo_det_list * dlp;
    struct scsiinfo_det_dev * ddp;
    struct scsiinfo_host_list * hlp;
    struct scsiinfo_host * hp;
    const int max_num = 10;
    int extra;
    size_t sz;
    char host_name[20];
    char host_info[82];

    si_fd = open("/dev/scsiinfo", O_RDONLY);
    if (si_fd < 0) {
    	perror("Problem opening /dev/scsiinfo");
	return 1;
    }
    res = ioctl(si_fd, SCSIINFO_GET_VERSION_NUM, &k);
    if (res < 0) {
    	perror("si_test ioctl SCSIINFO_GET_VERSION_NUM");
	return 1;
    }
    printf("scsiinfo version number=%d\n", k);
    res = ioctl(si_fd, SCSIINFO_GET_MX_DLIST_LEN, &k);
    if (res < 0) {
    	perror("si_test ioctl SCSIINFO_GET_MX_DLIST_LEN");
	return 1;
    }
    printf("scsiinfo max dlist len=%d\n", k);
    k = 20;
    res = ioctl(si_fd, SCSIINFO_SET_MX_DLIST_LEN, &k);
    if (res < 0) {
    	perror("si_test ioctl SCSIINFO_SET_MX_DLIST_LEN");
	return 1;
    }
    res = ioctl(si_fd, SCSIINFO_GET_MX_DLIST_LEN, &k);
    if (res < 0) {
    	perror("si_test ioctl SCSIINFO_GET_MX_DLIST_LEN");
	return 1;
    }
    printf("scsiinfo max dlist len=%d\n", k);
    res = ioctl(si_fd, SCSIINFO_GET_STATE, &ss);
    if (res < 0) {
    	perror("si_test ioctl SCSIINFO_GET_STATE");
	return 1;
    }
    printf("init time after boot [secs=%ld nsec=%ld] devs_att=%u, "
    	   "total_att=%u\n  total_det=%u, det_llen=%u, event_count=%lu, "
	   "lock_mod_state=%d\n  count_hosts=%d, high_host=%u\n", 
	   ss.init_time.tv_sec, ss.init_time.tv_nsec,
	   ss.devices_attached, ss.total_attachs, ss.total_detachs, 
	   ss.detach_list_len, ss.event_count, ss.lock_mod_state,
	   ss.count_hosts, ss.high_host);

    extra = 0;
    if (max_num > 1);
    	extra = max_num - 1;

    sz = sizeof(struct scsiinfo_att_list) + 
    	 (extra * (sizeof(struct scsiinfo_att_dev)));
    alp = (struct scsiinfo_att_list *)malloc(sz);
    if (NULL == alp) {
    	fprintf(stderr, "si_test memory 1\n");
	return 1;
    }
    memset(alp, 0, sz);
    alp->max_num = max_num;
    res = ioctl(si_fd, SCSIINFO_GET_ATT_LIST, alp);
    if (res < 0) {
    	perror("si_test ioctl SCSIINFO_GET_ATT_LIST");
	return 1;
    }
    printf("att_list: curr_event=%lu, [match_event=%lu, max_num=%d] "
    	   "num_out=%d\n", alp->curr_event, alp->match_event,
	   alp->max_num, alp->num_out);
    for ( k = 0; k < alp->num_out; ++k) {
    	adp = &alp->arr[k];
	printf("  attach time after boot [secs=%ld nsec=%ld] "
	       " attach_event=%lu\n", adp->att_time.tv_sec, 
	       adp->att_time.tv_nsec, adp->att_event);
	printf("    <%u, %u, %u, %u> vendor: %.8s, model: %.16s, rev: %.4s\n",
	       adp->d.host, adp->d.bus, adp->d.target, adp->d.lun, 
	       adp->d.vendor, adp->d.model, adp->d.rev);
	printf("    type=%d, scsi_level=%d, removable=%d, emulated=%d\n",
	       adp->d.scsi_type, adp->d.scsi_level, adp->d.removable,
	       adp->d.emulated);
    }
    free(alp);
    
    sz = sizeof(struct scsiinfo_det_list) + 
    	 (extra * (sizeof(struct scsiinfo_det_dev)));
    dlp = (struct scsiinfo_det_list *)malloc(sz);
    if (NULL == dlp) {
    	fprintf(stderr, "si_test memory 2\n");
	return 1;
    }
    memset(dlp, 0, sz);
    dlp->max_num = max_num;
    res = ioctl(si_fd, SCSIINFO_GET_DET_LIST, dlp);
    if (res < 0) {
    	perror("si_test ioctl SCSIINFO_GET_DET_LIST");
	return 1;
    }
    printf("\ndet_list: curr_event=%lu, [match_event=%lu, max_num=%d] "
    	   "num_out=%d\n", dlp->curr_event, dlp->match_event,
	   dlp->max_num, dlp->num_out);
    for ( k = 0; k < dlp->num_out; ++k) {
    	ddp = &dlp->arr[k];
	printf("  detached secs=%ld,nsec=%ld after boot,"
	       " attach_event=%lu, detach_event=%lu\n", 
	       ddp->det_time.tv_sec, ddp->det_time.tv_nsec, ddp->att_event,
	       ddp->det_event);
	printf("    <%u, %u, %u, %u> vendor: %.8s, model: %.16s, rev: %.4s\n",
	       ddp->d.host, ddp->d.bus, ddp->d.target, ddp->d.lun, 
	       ddp->d.vendor, ddp->d.model, ddp->d.rev);
	printf("    type=%d, scsi_level=%d, removable=%d, emulated=%d\n",
	       ddp->d.scsi_type, ddp->d.scsi_level, ddp->d.removable,
	       ddp->d.emulated);
    }
    free(dlp);
    
    sz = sizeof(struct scsiinfo_host_list) + 
    	 (extra * (sizeof(struct scsiinfo_host)));
    hlp = (struct scsiinfo_host_list *)malloc(sz);
    if (NULL == hlp) {
    	fprintf(stderr, "si_test memory 3\n");
	return 1;
    }
    memset(hlp, 0, sz);
    hlp->max_num = max_num;
    res = ioctl(si_fd, SCSIINFO_GET_HOST_LIST, hlp);
    if (res < 0) {
    	perror("si_test ioctl SCSIINFO_GET_HOST_LIST");
	return 1;
    }
    printf("\nhost_list: curr_event=%lu, [host=%u, flags=%d, max_num=%d] "
    	   "num_out=%d\n", hlp->curr_event, hlp->host, hlp->flags,
	   hlp->max_num, hlp->num_out);
    for ( k = 0; k < hlp->num_out; ++k) {
    	hp = &hlp->arr[k];
	strncpy(host_name, hp->host_name, sizeof(hp->host_name));
	host_name[sizeof(hp->host_name)] = '\0';
	printf("  host=%u, host_scsi_id=%d, emulated=%d, module=%d, "
	       "host_name: %s\n", hp->host, hp->host_scsi_id, hp->emulated,
	       hp->is_module, host_name);
	strncpy(host_info, hp->host_info, sizeof(hp->host_info));
	host_info[sizeof(hp->host_info)] = '\0';
	printf("    %s\n", host_info);
    }
    free(hlp);

    if (ss.lock_mod_state)
	ioctl(si_fd, SCSIINFO_UNLOCK_MOD, 0);
    else
	ioctl(si_fd, SCSIINFO_LOCK_MOD, 0);
    close(si_fd);
    return 0;
}
